#!/bin/bash
# Script written by Kathryn Cassidy, Trinity College Dublin
# for Grid-Ireland / EGEE tutorial March 2006

# Check args, exit with usage message if incorrect
if [ $1 ]; then
   echo "$1 jobs to submit";
else
    echo "Usage: ./submit-dictionary-jobs.sh <number-of-iterations>";
    exit 1;
fi

# Set the location of the files and dirs we will use
wordlist="/usr/share/dict/words"; # This is the unix dictionary wordlist
jdldir="$PWD/jdlfiles"; # This is where we will create the jdl files
statusdir="$PWD/status"; # This is where we'll put the output of commands we run
jobidfile="./jobid"; # This is the jobid file where the job ids will be stored.
jobopdir="$PWD/jobop"; # This is the directory to whhich output files are retireved
      # We'll pass this as a param to commands.
resultsfile="$PWD/results.txt"; # This is the temporary results file

if [ -f $resultsfile ]; then
    echo "Fatal error: results file $resultsfile already exists.";
    exit 1;
fi

# Set wait period, this is how long between each call to glite-wms-job-status when we
# are waiting for our jobs to complete.  Value is in seconds.
waitperiod=30;

# Create our dirs if they don't already exist
if [ ! -d $jdldir ]; then
    `mkdir $jdldir`;
    if [ $? != 0 ]; then
	echo "Fatal error: couldn't create $jdldir";
	exit 1;
    fi
fi
if [ ! -d $statusdir ]; then
    `mkdir $statusdir`;
    if [ $? != 0 ]; then
        echo "Fatal error: couldn't create $statusdir";
        exit 1;
    fi
fi


if [ ! -d $jobopdir ]; then
    `mkdir $jobopdir`;
    if [ $? != 0 ]; then
        echo "Fatal error: couldn't create $jobopdir";
        exit 1;
    fi
fi


# Iterate $1 times ($1 is the argument passed to our script)
for i in `seq 1 $1`; do

    # Get a random number between 1 and length of the wordlist file
    RANGE=45427;
    number=$RANDOM;
    let "number %= $RANGE";

    # Get the word on line $number from the wordlist file
    word=`head -n$number $wordlist | tail -n1`

    # Make a jdl file for this word
    echo "VirtualOrganisation = \"$2\";" >$jdldir/dictionary$i.jdl;
    echo "Executable = \"echoword.sh\";" >> $jdldir/dictionary$i.jdl;
    echo "Arguments = \"$i $word\";" >> $jdldir/dictionary$i.jdl;
    echo "StdOutput = \"std.out\";" >> $jdldir/dictionary$i.jdl; 
    echo "StdError = \"std.err\";" >> $jdldir/dictionary$i.jdl;
    echo "InputSandbox = \"echoword.sh\";" >> $jdldir/dictionary$i.jdl;
    echo "OutputSandbox = {\"std.out\",\"std.err\"};" >> $jdldir/dictionary$i.jdl;
    echo "ShallowRetryCount = 3;" >> $jdldir/dictionary$i.jdl;
    echo "RetryCount = 3;" >> $jdldir/dictionary$i.jdl;

    # Submit job
    `glite-wms-job-submit -a -o $jobidfile \
	$jdldir/dictionary$i.jdl 1> $statusdir/submit$i.out 2>$statusdir/submit$i.err`;

    # Check for errors
    if [[ $? != 0 ]]; then
	echo "Error reported by edg command";
	`cat $statusdir/submit$i.err`;
    else 
	echo "Job $i submitted, glite-wms-job-submit details are in file $statusdir/submit$i.out";
    fi

done

echo "All jobs submitted.  Job Ids for our jobs are stored in $jobidfile";

incomplete=1;
# Now we wait for the jobs to complete
while [[ $incomplete > 0 ]]; do

    # Wait a while between each iteration of this loop
    echo "Waiting $waitperiod seconds before checking status";
    sleep $waitperiod;

    # Check to see if our jobs are complete using edg-job-status
    `glite-wms-job-status --noint -i $jobidfile 1>$statusdir/jobstatus.out \
	2>$statusdir/jobstatus.err`;

    # Verify that we were able to get status, exit if we can't
    if [ $? != 0 ]; then
	echo "Fatal error occurred while getting job status.";
	echo "See file $statusdir/jobstatus.out for details.";
	exit 1;
   fi 

    # Read $statusdir/jobstatus.out, find the Current Status lines and
    # increment our incomplete counter if we find uncompleted jobs
    incomplete=0;
    while read line; do

	# See if line is a Current Status line
	result=`echo "${line}" | awk /"^Current Status:"/`;
	if [[ "$result" != "" ]]; then
	    # See if line is not in state Done, Cancelled, Aborted or Cleared
	    state=`echo $result | awk "! /Done|Cancelled|Aborted|Cleared/"`;
	    if [[ "$state" != "" ]]; then
	    	let incomplete=$incomplete+1; # increment incomplete counter
	    fi
	fi
    done < $statusdir/jobstatus.out;
echo "$incomplete job(s) incomplete";
done

# Get output for all of our jobs using glite-wms-job-output, no looping or
# waiting here, it doesn't return 'till output is retrieved.
`glite-wms-job-output --noint -i $jobidfile --dir $jobopdir  1>$statusdir/joboutput.out \
	2>$statusdir/joboutput.err`;

# Check for errors
if [[ $? != 0 ]]; then
    echo "Error reported by edg command";
    `cat $statusdir/jobstatus.err`;
else
    echo "Output retrieved, results are in file $jobopdir";
	ls -lR $jobopdir;
fi


# Finally get our output files and concatenate them into a file, then
# echo that to the screen with instructions for the user.
while read line; do
    if [ -d "${line}" ]; then
	`cat "${line}/std.out" >> $resultsfile`;
    fi
done < $statusdir/joboutput.out;

echo "Processing complete;";
echo "";
echo "";
echo "Please use each of the following words in a sentence:";
echo "--------------------------------------------------------------------------";
cat $resultsfile;
echo "--------------------------------------------------------------------------";
echo "Bye!";

`unlink $resultsfile`;
`unlink $jobidfile`;

exit 0;
